package com.pizza.entity;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 
 * @author 
 * Entity to store the order Information
 */
public class Order {
	/**
	 * Simple date Format object to format the date while parsing and Formatting
	 */
	private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	/**
	 * DecimalFormat to format the price 
	 */
	private static DecimalFormat df2 = new DecimalFormat(".##");
/**
 * Date on which order was made
 */
	private Date date;
	/**
	 * id of the customer ordering Pizza
	 */
	private int customerId;
	/**
	 * id of the ordered Pizza
	 */
	private String pizzaID;
	/**
	 * Cost of the pizza total
	 */
	private double cost;
	
	/**
	 * 
	 *getters and setters of the Class
	 */
	public Date getDate() {
		return date;
	}
	public void setDate(Date date) {
		this.date = date;
	}
	public int getCustomerId() {
		return customerId;
	}
	public void setCustomerId(int customerId) {
		this.customerId = customerId;
	}
	public String getPizzaID() {
		return pizzaID;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		long temp;
		temp = Double.doubleToLongBits(cost);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + customerId;
		result = prime * result + ((date == null) ? 0 : date.hashCode());
		result = prime * result + ((format == null) ? 0 : format.hashCode());
		result = prime * result + ((pizzaID == null) ? 0 : pizzaID.hashCode());
		return result;
	}
	
	/**
	 * Equals method to compare the similarity of two objects
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Order other = (Order) obj;
		if (Double.doubleToLongBits(cost) != Double.doubleToLongBits(other.cost))
			return false;
		if (customerId != other.customerId)
			return false;
		if (date == null) {
			if (other.date != null)
				return false;
		} else if (!date.equals(other.date))
			return false;
		if (format == null) {
			if (other.format != null)
				return false;
		} else if (!format.equals(other.format))
			return false;
		if (pizzaID == null) {
			if (other.pizzaID != null)
				return false;
		} else if (!pizzaID.equals(other.pizzaID))
			return false;
		return true;
	}
	public void setPizzaID(String pizzaID) {
		this.pizzaID = pizzaID;
	}
	public double getCost() {
		return cost;
	}
	public void setCost(double cost) {
		this.cost = cost;
	}
	/**
	 * 
	 * @param date order date
	 * @param customerId Id of customer ordering Pizza
	 * @param pizzaID id of the Pizza
	 * @param cost Total cost of the Pizza
	 * Initializes the instance variables of the Class
	 */
	public Order(Date date, int customerId, String pizzaID, double cost) {
		super();
		this.date = date;
		this.customerId = customerId;
		this.pizzaID = pizzaID;
		this.cost = cost;
	}
	@Override
	public String toString() {
		return format.format(date) + "," + customerId + "," + pizzaID
				+ "," + df2.format(cost);
	}
	
}
