package com.pizza.service;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Scanner;

import com.pizza.entity.Base;
import com.pizza.entity.Order;
import com.pizza.entity.Size;
import com.pizza.entity.Topping;
import com.pizza.entity.User;

/**
 * 
 * @author 
 * Class with all Pizza ordering functionality
 *
 */
public class MainFunction {
	/**
	 * decimal format object to format double values
	 */
	private static DecimalFormat df2 = new DecimalFormat(".##");
	/**
	 * Scanner object to take inputs
	 */
	private Scanner sc=new Scanner(System.in);
	/**
	 * Logged in User
	 */
	private User user;
	/**
	 * Format date and Parse it
	 */
	private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	/**
	 * List of orders
	 */
	private List<Order> orders;
	/**
	 * List of available sizes
	 */
	private List<Size> sizes;
	/**
	 * List of availabe bases
	 */
	private List<Base> bases;
	/**
	 * List of available Toppings
	 */
	private List<Topping> toppings;

	/**
	 * File Names 
	 */
	private static String baseFile="base.txt";
	private static String toppingFile="toppings.txt";
	private static String sizeFile="size.txt";
	private static String orderFile="orders.txt";

	/**
	 * 
	 * @param user current logged in user
	 * Read all the details of sizes, bases, orders and topping sinto their corresponding lists
	 */
	public MainFunction(User user) {
		this.user=user;
		readOrders();
		readToppings();
		readSizes();
		readBases();
	}
	/**
	 * read the bases file in its ArrayList
	 */
	private void readBases() {
		bases=new ArrayList<>();
		try (BufferedReader br = new BufferedReader(new FileReader(baseFile))) {
			String line;
			while ((line = br.readLine()) != null) {
				String[] parts=line.split(",");
				Base base=new Base(Integer.parseInt(parts[0]), parts[1], Double.parseDouble(parts[2]));
				bases.add(base);	
			}
		} catch (IOException e) {
			e.printStackTrace();
			System.out.println("Error in opening Bases File.Exiting...");
			System.exit(0);	
		}	
	}
	/**
	 * read the sizes file in its ArrayList
	 */
	private void readSizes() {
		sizes=new ArrayList<>();
		try (BufferedReader br = new BufferedReader(new FileReader(sizeFile))) {
			String line;
			while ((line = br.readLine()) != null) {
				String[] parts=line.split(",");
				Size base=new Size(Integer.parseInt(parts[0]), parts[1], Double.parseDouble(parts[2]));
				sizes.add(base);	
			}
		} catch (IOException e) {
			e.printStackTrace();
			System.out.println("Error in opening Sizes File.Exiting...");
			System.exit(0);	
		}

	}
	/**
	 * read the toppings file in its ArrayList
	 */
	private void readToppings() {
		toppings=new ArrayList<>();
		try (BufferedReader br = new BufferedReader(new FileReader(toppingFile))) {
			String line;
			while ((line = br.readLine()) != null) {
				String[] parts=line.split(",");
				Topping topping=new Topping(Integer.parseInt(parts[0]), parts[1], Double.parseDouble(parts[2]));
				toppings.add(topping);	
			}
		} catch (IOException e) {
			e.printStackTrace();
			System.out.println("Error in opening Toppings File.Exiting...");
			System.exit(0);	
		}


	}

	/**
	 * read the orders file in its ArrayList
	 */
	private void readOrders() {
		orders=new ArrayList<>();
		try (BufferedReader br = new BufferedReader(new FileReader(orderFile))) {
			String line;
			while ((line = br.readLine()) != null) {
				String[] parts=line.split(",");
				Order order=null;
				try {
					order = new Order(format.parse(parts[0]),Integer.parseInt(parts[1]),
							parts[2],Double.parseDouble(parts[3]));
				} catch (NumberFormatException | ParseException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				orders.add(order);	
			}
		} catch (IOException e) {
			e.printStackTrace();
			System.out.println("Error in opening Toppings File.Exiting...");
			System.exit(0);	
		}		
	}

	/**
	 * the customer is taken through a four-stage process of ordering a pizza.
	 * At any stage of the process the customer is	 able to exit and return to the
	 * main menu.
	 */
	public void orderPizza() {
		System.out.println("Select details about your Pizza");
		Base base=selectBase();
		if(base==null)
		{		
			System.out.println("Exiting to Main menu.");
			return ;
		}
		List<Topping> pTop=new ArrayList<>();
		System.out.println(base.getBase());
		while(true)
		{
			Topping topping=selectTopping();
			if(topping==null)
			{
				System.out.println("Exiting to Main menu.");
				return;
			}
			pTop.add(topping);
			System.out.println(topping.getTopping());
			if(pTop.size()==5)
			{
				System.out.println("5 topping added");
				break;
			}

			System.out.print("Do you want to add another Topping(y/n) : ");
			char ch=' ';
			while(true)
			{
				ch=sc.nextLine().charAt(0);
				if(ch=='n' || ch=='N' || ch=='y' || ch=='Y')
					break;
				else
					System.out.println("Please enter only y/n");
			}
			if(ch=='n' || ch=='N')
				break;

		}
		double cost=0.0;
		cost+=base.getPrice();
		for(Topping topping:pTop)
			cost=cost+topping.getPrice();
		Size size=getSize(cost);
		if(size==null)
		{
			System.out.println("Exiting to Main menu.");
			return;
		}
		System.out.print("Do you confirm your order (y/n) : ");
		char ch=' ';
		while(true)
		{
			ch=sc.nextLine().charAt(0);
			if(ch=='n' || ch=='N' || ch=='y' || ch=='Y')
				break;
			else
				System.out.println("Please enter only y/n");
		}
		if(ch=='n' || ch=='N')
			return;
		Date date=new Date();
		int customerId=user.getId();
		String pid="P"+date.getTime();
		cost=cost+size.getMultiplier();
		Order order=new Order(date, customerId, pid, cost);
		orders.add(order);
		saveOrder();
		StringBuilder sb=new StringBuilder("");
		sb.append(size.getId()).append("\n");
		sb.append(base.getId()).append("\n");
		for(int i=0;i<pTop.size();i++)
		{
			Topping top=pTop.get(i);
			if(i==pTop.size()-1)
				sb.append(top.getId()).append("\n");
			else
				sb.append(top.getId()).append(",");
		}
		savePizzaFile(pid+"-"+customerId+".txt",sb.toString());
		System.out.println("Order confirmed successfully and will be delivered in 30 minutes");

	}

	/**
	 * 
	 * @param file name of file
	 * @param content content to be saved
	 * save the passed String content to file
	 */
	private void savePizzaFile(String file, String content) {
		try (BufferedWriter bw = new BufferedWriter(new FileWriter(file))) {
			bw.write(content);
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	/**
	 * save the changes made to Orders List into File
	 */
	private void saveOrder() {
		try (BufferedWriter bw = new BufferedWriter(new FileWriter(orderFile))) {
			String content = getOrderData();
			bw.write(content);
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	/**
	 * 
	 * @return all the order data as String
	 */
	private String getOrderData() {
		StringBuilder sb=new StringBuilder("");
		for(int i=0;i<orders.size();i++)
		{
			Order order=orders.get(i);
			if(i==orders.size()-1)
				sb.append(order);
			else
				sb.append(order).append("\n");

		}
		return sb.toString();
	}
	/**
	 * 
	 * @return a size selected for the Pizza
	 * null if user want to exit
	 */
	private Size getSize(double cost) {
		while(true)
		{
			System.out.println("Select a Size : ");
			int i=1;
			for(Size size:sizes)
				System.out.println(i++ +"\tSize : "+size.getSize()+"\t\tTotal Cost Price : "+df2.format(size.getMultiplier()*cost));
			System.out.println("0 : Exit");
			int ch=Integer.parseInt(sc.nextLine());
			if(ch < 0 && ch>sizes.size())
				System.out.println("Invalid Choice. Try Again.");
			else
				return ch==0?null:sizes.get(ch-1);

		}
	}
	/**
	 * 
	 * @return a Base selected for the Pizza
	 * null if user want to exit
	 */
	private Base selectBase() {
		while(true)
		{
			System.out.println("Select a Base : ");
			int i=1;
			for(Base base:bases)
				System.out.println(i++ +"\tBase : "+base.getBase()+"\t\tPrice : "+df2.format(base.getPrice()));
			System.out.println("0 : Exit");
			int ch=Integer.parseInt(sc.nextLine());
			if(ch < 0 && ch>bases.size())
				System.out.println("Invalid Choice. Try Again.");
			else
				return ch==0?null:bases.get(ch-1);

		}
	}

	/**
	 * 
	 * @return a topping selected for the Pizza
	 * null if user want to exit
	 */
	private Topping selectTopping() {
		while(true)
		{
			System.out.println("Select a Topping : ");
			int i=1;
			for(Topping topping:toppings)
				System.out.println(i++ +"\tTopping : "+topping.getTopping()+"\t\tPrice : "+df2.format(topping.getPrice()));
			System.out.println("0 : Exit");
			int ch=Integer.parseInt(sc.nextLine());
			if(ch < 0 && ch>toppings.size())
				System.out.println("Invalid Choice. Try Again.");
			else
				return ch==0?null:toppings.get(ch-1);

		}
	}

	/**
	 * Cancel an order
	 * Upon selecting a pizza, the order is immediately cancelled, which requires
	 *	• The appropriate entry to be removed from the orders file
	 *  • The relevant pizza file to be deleted
	 */
	public void cancelOrder() {
		List<Order> myorders=getUserOrders();
		List<Order> validOrders=getValidOrders(myorders);
		System.out.println("Select an Order to Remove : ");
		int i=1;
		if(validOrders.isEmpty())
		{
			System.out.println("No orders found ordered in last 10 minutes.");
			return;
		}
		for(Order order:validOrders)
		{
			System.out.println(i++ + " Order : "+order.getPizzaID()+"-"+df2.format(order.getCost()));
		}
		int ch=Integer.parseInt(sc.nextLine());
		if(ch < 1 && ch>validOrders.size())
		{
			System.out.println("Invalid Choice. ");
			return ;
		}
		Order order=validOrders.get(ch-1);
		orders.remove(order);
		saveOrder();
		String fileName=order.getPizzaID()+"-"+order.getCustomerId()+".txt";
		File file=new File(fileName);
		file.delete();
		System.out.println("Order cancelled successfully");




	}

	/**
	 * 
	 * @param myorders list of the orders of the Customer
	 * @return the orders ordered in last 10 minutes
	 */
	private List<Order> getValidOrders(List<Order> myorders) {
		List<Order> vOrders=new ArrayList<>();
		Date date=new Date();

		for(Order order:myorders)
		{
			long seconds = (date.getTime()-order.getDate().getTime())/1000;
			if(seconds<=600)
				vOrders.add(order);

		}
		return vOrders;
	}

	/**
	 * 
	 * @return list of orders of Customer from list of all orders
	 */
	private List<Order> getUserOrders()
	{
		List<Order> myorders=new ArrayList<>();
		for(Order order:orders)
		{
			if(order.getCustomerId()==user.getId())
				myorders.add(order);
		}
		return myorders;
	}

	/**
	 * Vies all the order details of the Customer
	 */
	public void viewOrders() {
		System.out.println("Orders for "+user.getForeName());
		List<Order> myorders=getUserOrders();
		int i=1;
		if(myorders.isEmpty())
		{
			System.out.println("No Order exist for "+user.getForeName());
		}
		System.out.println(String.format("%-25s %-10s %-10s %-10s", "Date","Size","Base","Price"));
		String size="",base="";
		for(Order order:myorders)
		{
			String fileName=order.getPizzaID()+"-"+order.getCustomerId()+".txt";

			try (BufferedReader br = new BufferedReader(new FileReader(fileName))) {
				size=br.readLine();
				base=br.readLine();

			} catch (IOException e) {
				e.printStackTrace();
			}	
			System.out.println(String.format("%-25s %-10s %-10s %-10s", format.format(order.getDate()),size,base,df2.format(order.getCost())));

		}

	}

	/**
	 * Edit registration details for the user
	 */
	public void editRegistration() {
		System.out.println("Enter new Email id :");
		String email=sc.nextLine();
		System.out.println("Enter new Password : ");
		String pwd=sc.nextLine();	
		user.setEmail(email);
		user.setPassword(pwd);
		UserAuthentication usA=new UserAuthentication();
		usA.updateUserDetails(user);
		System.out.println("User registration updated successfully.");
	}

}
