package com.pizza.service;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import com.pizza.entity.User;

/**
 * 
 * @author 
 * Class to handle all the login and registration related functionality
 *
 */
public class UserAuthentication {
	/**
	 * List of users in the File
	 */
	private List<User> users;
	/**
	 * Name of the Users File
	 */
	public static String usersFile="registration.txt";

	/**
	 * read the users to corresponding ArrayList
	 */
	public UserAuthentication()
	{
		users=getUsersFromFile();
	}

	/**
	 * 
	 * @return arrayList of Users
	 * read the Users File and return the ArrayList containing them
	 */
	private List<User> getUsersFromFile() {
		List<User> tUsers=new ArrayList<>();
		try (BufferedReader br = new BufferedReader(new FileReader(usersFile))) {

			String line;

			while ((line = br.readLine()) != null) {
				String[] parts=line.split(",");
				User user=new User(Integer.parseInt(parts[0]),parts[1],parts[2],parts[3],parts[4]);
				tUsers.add(user);	
			}

		} catch (IOException e) {
			e.printStackTrace();
			System.out.println("Error in opening Registration File.Exiting...");
			System.exit(0);

		}
		return tUsers;
	}

	/**
	 * 
	 * @param email email of user
	 * @param pwd pwd of user
	 * @return user if user with email and pwd combo exists
	 * else return null
	 */
	public User authenticate(String email, String pwd) {
		for(User user:users)
		{
			if(email.equals(user.getEmail()) && pwd.equals(user.getPassword()))
				return user;
		}
		return null;
	}

	/**
	 * 
	 * @param user user to be added to list of users
	 * Save the changes to file also
	 */
	public void registerUser(User user) {
		int id=generateId();
		user.setId(id);
		users.add(user);

		users.sort(new Comparator<User>() {

			@Override
			public int compare(User o1, User o2) {
				return o1.getId()-o2.getId();
			}
		});
		saveFile();

	}
	/**
	 * save the Users detail to File
	 */
	public void saveFile()
	{
		try (BufferedWriter bw = new BufferedWriter(new FileWriter(usersFile))) {
			String content = getFileData();
			bw.write(content);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	/**
	 * 
	 * @return the Users detail as String
	 */
	private String getFileData() {
		StringBuilder sb=new StringBuilder("");
		for(int i=0;i<users.size();i++)
		{
			User user=users.get(i);
			if(i==users.size()-1)
				sb.append(user);
			else
				sb.append(user).append("\n");

		}

		return sb.toString();
	}

	/**
	 * 
	 * @return the generated id for the user
	 */
	private int generateId() {
		int id=0;
		for(User user:users)
		{
			if(user.getId()>id)
				id=user.getId();
		}
		return id+1;
	}

	/**
	 * 
	 * @param u details of the user to be updated
	 * update the details and save changes to file
	 */
	public void updateUserDetails(User u) {
		for(User user:users)
		{
			if(user.getId()==u.getId())
			{
				user.setEmail(u.getEmail());
				user.setPassword(u.getPassword());
				saveFile();
				return;
			}
		}

	}
}
